import os
from StringIO import StringIO
from OFS.ObjectManager import BadRequestException
from Products.CMFCore import permissions
from Products.ZCatalog.ZCatalog import manage_addZCatalog
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.config import REFERENCE_CATALOG
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import WorkflowPolicyConfig_id
from Products.CMFEditions.Permissions import RevertToPreviousVersions

from Products.Intranet import config
from Products.Intranet.utils import installDepends
from Products.Intranet.utils import add_form_controller_overrides
from Products.Intranet.utils import remove_form_controller_overrides

from zope.interface import directlyProvides, directlyProvidedBy
#from Products.Intranet.Extensions.utils import setupKupu

def fixUpEditTab(portal, out):
    pt=getToolByName(portal, 'portal_types')
    action=pt.Document._actions[1]
    action.permissions=(permissions.View,)
    print >> out, "edit tabs set to view permission"
    
def installColumns(portal, out):
    catalog = getToolByName(portal, 'portal_catalog')
    switch=dict([(x, True) for x in catalog.schema()])
    #add = catalog.addColumn
    #[add(col) for col in cols\
    # if not switch.has_key(col) ]
    
    print >> out, 'metadata columns %s installed'

def setupSitePortlets(portal, out):
    print >> out, 'Portlet configuration:'

    print >> out, '-> Removing all site portlets'
    portal.manage_changeProperties(left_slots=tuple())
    portal.manage_changeProperties(right_slots=tuple())

    mdc = getToolByName(portal, 'portal_memberdata')
    print >> out, '-> Removing all portal_memberdata portlets'
    #mdc.manage_changeProperties(left_slots=tuple())
    mdc.manage_changeProperties(right_slots=tuple())

def navTreeSetup(portal, out):
    nav_types = config.NAVTREE_TYPES
    portal.prefs_navigation_set(generated_tabs=False, portaltypes=nav_types)
    print >> out, '-> Navigation Tree types set'

def setupPortalActions(portal, out):
    print >> out, '-> Changing condition on the "home" portal tab'
    atool = getToolByName(portal, 'portal_actions')
    action = atool.getActionObject('portal_tabs/index_html')
    #action.edit(title="Intranet Home",
    #                 visible=False)

    mtool = getToolByName(portal, 'portal_membership')
    #action = mtool.getActionObject('user/mystuff')
    #action.edit(title='My Home')

    action = mtool.getActionObject('user/preferences')
    action.edit(title='My Info',
                action="string:${portal_url}/portal_memberdata/${member/id}/edit")

    '''print >> out, '-> Adding "About" portal tab'
    if atool.getActionObject('portal_tabs/about') is None:
        atool.addAction('about', 'About',
                        'string:${portal_url}/about',
                        "python:True",
                        'View', 'portal_tabs')

    print >> out, '-> Adding "projects" portal tab'
    if atool.getActionObject('portal_tabs/projects') is None:
        atool.addAction('projects', 'Projects',
                        'string:${portal_url}/projects',
                        "python:True",
                        'View', 'portal_tabs')'''

    print >> out, '-> Adding "forum" portal tab'
    if atool.getActionObject('portal_tabs/forum') is None:
        atool.addAction('forum', 'Forum',
                    'string:$portal_url/forum',
                    '',
                    'View', 'portal_tabs', True)

    '''print >> out, '-> Adding "news" portal tab'
    atool.addAction('korys_news', 'News',
                    'string:$portal_url/korys-news',
                    '',
                    'View', 'portal_tabs', True)

    print >> out, '-> Adding "statistics" portal tab'
    if atool.getActionObject('portal_tabs/statistic') is None:
        atool.addAction('statistic', 'Statistic',
                        'string:${portal_url}/stats',
                        "python:True",
                        'View', 'portal_tabs', True)

    print >> out, '-> Adding "add project" portal tab'
    if atool.getActionObject('portal_tabs/add_project') is None:
        atool.addAction('add_project', 'Start a Project',
                        'string:${portal_url}/projects/add_project',
                        "python:not portal.portal_membership.isAnonymousUser()",
                        'View', 'portal_tabs') '''

def hideActions(portal, out):
    actions_map = {#'portal_teams': ('user/myteamspaces',),
                   'portal_membership': ('user/mystuff', 'user/myvotes'),
                   'portal_actions': ('folder_buttons/change_state',
                                      'document_actions/full_screen',),
                   'portal_undo': ('user/undo',)
                   }
    for tool_id in actions_map.keys():
        tool = getToolByName(portal, tool_id)
        for action_id in actions_map[tool_id]:
            action = tool.getActionObject('%s' % action_id)
            if action is not None and action.visible:
                print >> out, "-> hiding '%s' action" % action_id
                action.edit(visible=False)

def hidePlonePortlets(portal, out):
    """
    we depend on PlonePortlets, but we don't actually use it to manage
    our site portlets yet, so we need to hide some of the UI
    """
    atool = getToolByName(portal, 'portal_actions')
    action = atool.getActionObject('object/assignportlets')
    if action is not None:
        print >> out, "Hiding 'Assign Portlets' action"
        action.edit(visible=False)

def hideActionTabs(portal, out):
    """
    we need to iterate through all of the portal types to hide the
    certain tabs... *sigh*
    """
    ttool = getToolByName(portal, 'portal_types')
    ftis = ttool.listTypeInfo()
    hidden_actions = ('object/local_roles', 'object/metadata')
    for fti in ftis:
        for act_id in hidden_actions:
            print >> out, "Hiding '%s' action for all types" % act_id
            action = fti.getActionObject(act_id)
            if action is not None:
                action.edit(visible=False)

def addProjectConfigAction(portal, out):
    """
    We want a special action available on projects and their default pages
    for configuring the project itself.
    """
    print >> out, "Moved project edit action to top"

def addUndeleteAction(portal, out):
    """
    We want a special action available on default pages of UndeleteContainers
    for undeleting objects.
    """
    atool = getToolByName(portal, 'portal_actions')
    action = atool.getActionObject('object/project_default_undelete')
    if action is None:
        atool.addAction("project_default_undelete",
                        "Undelete",
                        "string:$folder_url/undelete_view",
                        "python: (object.isDefaultPageInFolder() and "
                        "path('folder/@@undelete_view/listDeleted|nothing')) or "
                        "(object is folder and "
                        "path('object/@@undelete_view/listDeleted|nothing'))",
                        RevertToPreviousVersions,
                        "object")
        print >> out, "Adding undelete tab to project home page"

def uiTweaks(portal, out):
    print >> out, 'Applying site UI customizations'

    #setupSitePortlets(portal, out)

    print >> out, '-> Removing not addable types'
    ttool = getToolByName(portal, 'portal_types')
    for type in config.NOT_ADDABLE_TYPES:
        fti = ttool.getTypeInfo(type)
        if fti is not None:
            fti.global_allow = False

    #hideActions(portal, out)
    #navTreeSetup(portal, out)
    setupPortalActions(portal, out)
    #hidePlonePortlets(portal, out)
    #hideActionTabs(portal, out)
    addProjectConfigAction(portal, out)
    addUndeleteAction(portal, out)

def addprojectlistingview(portal, out):
    print >> out, 'Added project listing view'

def setupVersioning(portal, out):
    versioned_types = ('Document', 'Event', 'News Item', 'Link', 'File',
                      'Image')
    ttool = getToolByName(portal, 'portal_types')
    version_tool = getToolByName(portal, 'portal_repository', None)
    diff_tool = getToolByName(portal, 'portal_diff', None)
    if version_tool is not None:
        version_tool.setVersionableContentTypes(versioned_types)
    for p_type in versioned_types:
        if version_tool is not None:
            version_tool.addPolicyForContentType(p_type,
                                                    'at_edit_autoversion')
            version_tool.addPolicyForContentType(p_type,
                                                    'version_on_rollback')
        if diff_tool is not None:
            diff_tool.manage_addDiffField(p_type, 'none',
                                             'Compound Diff for AT types')
        print >> out, 'Enabled versioning for %s'%p_type
    if diff_tool is not None:
        ttool.ChangeSet.global_allow = False

def addForum(portal, out):
    # Main Forum added there
    if not 'forum' in portal.objectIds():
        print >> out, 'Creating Intranet Forum'
        forum = getToolByName(portal, 'portal_types')
        forum.constructContent('Ploneboard', portal, 
                               'forum', title='Forum')
        # Add last forum posts portlet.
        folder = portal._getOb('forum')
        #folder.manage_addProperty(id='left_slots', type='lines', value=('here/portlet_forum_recent/macros/portlet'))   

def addStatsFolder(portal, out):
    """
    Create Stats Folder. This is folder that will view portal statistics.
    """
    folder_id = 'stats'
    if folder_id not in portal.objectIds():        
        print >> out, '-> creating stats folder'
        portal.invokeFactory('Stats', folder_id, title = "Statistika", description = "Korio statistika")
        folder = portal._getOb(folder_id)
        #folder.setLayout('folder_listing')
        
        print >> out, '-> creating stats folder portlets'
        folder.manage_addProperty(id='left_slots', type='lines', value=('here/member_search_portlet/macros/portlet'))
        folder.manage_addProperty(id='right_slots', type='lines', value=('here/stats_portlet/macros/portlet'))
        
        # Add type restrictions
        #print >> out, 'Restricting addable types in About Folder'
        #folder.setConstrainTypesMode(1)
        #folder.setLocallyAllowedTypes(["Folder", "Document", "FAQ"])

def registerCSS(portal, out):
    print >> out, 'Registering style sheets'
    cssreg = getToolByName(portal, 'portal_css')
    cssreg.registerStylesheet('intranet_style.css')
    '''cssreg.registerStylesheet('ts.css')
    cssreg.registerStylesheet('ts_portlets.css')
    cssreg.registerStylesheet('openplans_print.css',
                              media="print")'''

def migrateATDocToWickedDoc(portal, out):
    print >> out, 'Migrating ATDocument type to WickedDoc'
    if not 'migrate_atdoc_wickeddoc' \
        in portal.objectIds(spec='External Method'):
        portal.manage_addProduct['ExternalMethod'].manage_addExternalMethod(
            'migrate_atdoc_wickeddoc', '', 'wicked.migrate_atdoc_wickeddoc',
            'migrate_atdoc_wickeddoc')

    import transaction as txn
    txn.commit(1)
    print >> out, portal.migrate_atdoc_wickeddoc(portal)
    portal.manage_delObjects(ids=['migrate_atdoc_wickeddoc'])

def setAboutKorysTopic(portal, out):
    folder = portal._getOb('about')
    
    print >> out, '-> creating „About Korys“ topic'
    topic_id = 'about_korys'
    folder.invokeFactory('Document', topic_id, title = "About Korys")
    folder.setDefaultPage(topic_id)    # setting default page in about folder
    topic = folder._getOb(topic_id)
    topic.setText('Čia reikia parašyti tekstą, tačiau reikia kažkaip padaryti su vertimais kas ir kaip \
                    privalo būti.')
    #state_crit = topic.addCriterion('review_state', 'ATSimpleStringCriterion')
    #state_crit.setValue('visible')
    
    # Set About permission
    
    perms = ('Modify portal content',)
    for perm in perms:
        topic.manage_permission(perm, roles=('Authenticated'))
    

def setSiteIndexPage(portal, out):
    topic_id = 'site-home'
    if topic_id not in portal.objectIds():        
        print >> out, '-> creating site index page'
        portal.invokeFactory('Topic', topic_id, title = "'Korio' naujienos", description = "")
        topic = portal._getOb(topic_id)

        type_crit = topic.addCriterion('Type','ATPortalTypeCriterion')
        type_crit.setValue('News Item')
        sort_crit = topic.addCriterion('created','ATSortCriterion')
        sort_crit.setReversed(True) 
        state_crit = topic.addCriterion('review_state', 'ATSimpleStringCriterion')
        state_crit.setValue('visible')

        news = getToolByName(portal, 'korys-news')
        path_crit = topic.addCriterion('path','ATPathCriterion')
        path_crit.setValue((news.UID(),))  #in news folder recursively
        path_crit.setRecurse(1)
        topic.portal_workflow.doActionFor(topic, 'publish', comment='') # making publish
        topic.setLayout('folder_summary_view')
        
        print >> out, '-> Adding right and left portlets for main page'
        rslot = ("here/portlet_newest_projects/macros/portlet", "here/portlet_active_user/macros/portlet")
        lslot = ("here/portlet_login/macros/portlet", "here/portlet_projects_news/macros/portlet", "here/portlet_recent/macros/portlet") 
        topic.manage_addProperty("right_slots", rslot, "lines")
        topic.manage_addProperty("left_slots", lslot , "lines")

        portal.setDefaultPage(topic_id)

def setCategories(portal, out):
    """Setup project categories
    """
    print >> out, '-> creating initial project categories'
    ttool = getToolByName(portal, 'portal_types')
    ctool = getToolByName(portal, 'portal_projectmap')
    utils = getToolByName(portal, 'plone_utils')
    def createCategories(root, catdefs, special=False):
        for cat in catdefs:
            newid = utils.normalizeString(cat['title'])
            catspecial = cat.get('special', special)
            at_type = catspecial and 'SpecialProjectCategory' or 'ProjectCategory'
            catundef = cat.get('undefined', False)
            if catundef:
                at_type = 'UndefinedCategory'
            ttool.constructContent(at_type, root, newid, title=cat['title'])

            created = root._getOb(newid)
            if cat.has_key('children'):
                createCategories(created, cat['children'], catspecial)

    createCategories(ctool, config.INITIAL_CATEGORIES)
    

def install(self, migrate_atdoc_to_wickeddoc=True):
    out = StringIO()
    portal = getToolByName(self, 'portal_url').getPortalObject()
    installDepends(self)
    install_subskin(self, out, config.GLOBALS)
    installTypes(portal, out, listTypes(config.PROJECTNAME), config.PROJECTNAME)
    #uiTweaks(portal, out)
    #addStatsFolder(portal, out)
    addForum(portal, out)
    registerCSS(portal, out)
    #setupKupu(portal, out)
    #installColumns(portal, out)
    #if migrate_atdoc_to_wickeddoc: # needed for unit test sanity
    #    migrateATDocToWickedDoc(portal, out)
    #setupVersioning(portal, out)
    #fixUpEditTab(portal, out)
    #setSiteIndexPage(portal, out)
    #setCategories(portal, out)

    print >> out, "Successfully installed %s." % config.PROJECTNAME
    
    # create_test_content(self)
    
    return out.getvalue()
